//Windows header
#ifdef WIN32
#include <windows.h>
#endif


// OpenGL header
#include "gl.h"
#include "glut.h"

#include <math.h>
#ifndef M_PI
	#define M_PI 3.14159265358979323846
#endif

#include "dynamics.h"

////////////////////////////////////
//  Dynamics :
////////////////////////////////////

Dynamics dynamics;


////////////////////////////////////
//  Camera :
////////////////////////////////////
float cameraAngle=M_PI*0.5f;
float distToOrigin = 27.0;
float cameraHeight = 6.0f;
void setCamera() {
	float x = distToOrigin*cos(cameraAngle);
	float y = cameraHeight;
	float z = distToOrigin*sin(cameraAngle);
	glLoadIdentity();
	gluLookAt(x, y, z, -x, y, -z, 0.0f,1.0f,0.0f);
}

////////////////////////////////////
//  Dessin :
////////////////////////////////////
void displayScene() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	dynamics.drawDynamics();
	dynamics.animate();

	glutSwapBuffers();
}

void idleScene() {
}


////////////////////////////////////
//  Gestion d'vnements :
////////////////////////////////////
void specialKey(int key, int x_, int y_) {
	switch (key) {
        case GLUT_KEY_LEFT :
            cameraAngle += 0.1f;
            setCamera();  
			break;
        case GLUT_KEY_RIGHT :
			cameraAngle -=0.1f;
			setCamera();  
			break;
        case GLUT_KEY_UP :
            distToOrigin -= 0.2f;
            if (distToOrigin<1.5f) distToOrigin = 1.5f;
            setCamera();  
			break;
        case GLUT_KEY_DOWN :
            distToOrigin += 0.2f;
            setCamera();  
			break;
        case GLUT_KEY_PAGE_UP :
            cameraHeight += 0.1f;
            setCamera();  
			break;
        case GLUT_KEY_PAGE_DOWN :
            cameraHeight -= 0.1f;
            setCamera();  
			break;
	}
}

void commonKey(unsigned char key, int x_, int y_) {
	switch(key) {
		case 27 :
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT | GL_ACCUM_BUFFER_BIT);
			exit(0);
		default :
			return;
	}
}


int prevX = 0, prevY = 0;
bool dragging = false;
float normDist = 1000.0f;
Vec initPos;

void mouseClick(int button, int state, int x,int y) {
	if (button == GLUT_LEFT_BUTTON && state==GLUT_DOWN){
				dragging = true;
				prevX = x;
				prevY = y;
		} else if (button == GLUT_LEFT_BUTTON && state==GLUT_UP){
				dragging = false;				
		}
}


void mouseMotion(int x,int y) {
	if (dragging) {
		float dx = (x-prevX)*distToOrigin/normDist;
		float dy = (y-prevY)*distToOrigin/normDist;


		initPos.x += sin(cameraAngle)*dx;
		initPos.z -=  cos(cameraAngle)*dx;
		initPos.y -= dy;
				
		dynamics.setMassPos(0,initPos);

		prevX = x;
		prevY = y;
	}
}

void changeSize(int w, int h)   {
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	
	float ratio = 1.0f * w / h;
	glViewport(0, 0, w, h);
	gluPerspective(45,ratio,1,1000);
	glMatrixMode(GL_MODELVIEW);
	setCamera();
}


////////////////////////////////////
//  Mthodes d'initialisation :
////////////////////////////////////

// Initialisation :
void initScene(void) {
	// init GL states
	glClearDepth(1.0);	
	glClearColor(1.0, 1.0, 1.0, 1.0);							

	glDepthFunc(GL_LESS);
	glEnable(GL_DEPTH_TEST);

	glShadeModel(GL_SMOOTH);
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	GLfloat lightAmbient[]= { 0.2f, 0.2f, 0.2f, 1.0f };
	GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
	float lightAngle = 0.0f;
	GLfloat lightPosition[]= { 4.0*cos(lightAngle), 3.0f, 4.0*sin(lightAngle), 1.0f };
    glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_POSITION,lightPosition);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	glEnable(GL_COLOR_MATERIAL);

	dynamics.initDynamics();
	initPos = dynamics.getMassPos(0);
}

////////////////////////////////////
//  Main method
////////////////////////////////////
int main(int argc, char **argv) {
	// crer la fentre
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(800, 600);
	glutCreateWindow("Dynamics");

	// Evnements clavier/souris :
	glutSpecialFunc(specialKey);
	glutKeyboardFunc(commonKey);
	glutMouseFunc(mouseClick);
	glutMotionFunc(mouseMotion);
	
	// Mthodes de rendu :
	glutDisplayFunc(displayScene);
	glutIdleFunc(displayScene);

	// Changement de taille de la fentre :
	glutReshapeFunc(changeSize);

	// Initialise OpenGL :
	initScene();

	// run program
	glutMainLoop();

	return(0);
}
