#include "dynamics.h"

#include "solver.h"

//Windows header
#ifdef WIN32
#include <windows.h>
#endif
// OpenGL header
#include "gl.h"
#include "glut.h"

// Cre le systme dynamique :
void Dynamics::initDynamics() {
	///////////////////////////////////////
	//////////// TO MODIFY ////////////////
	///////////////////////////////////////


	////////////////////////////////////
	// 1D : Chane articule :
	////////////////////////////////////
	Vec initPosA (0.0, 5.0, 0.0);
	float massA = 1.0;
	float radiusA = 0.25;
	float stiffnessA = 60.0;
	float initLengthA = 5.0;
	float dampingA = 0.2;
	// add a fixed ball
	int mass0 = addMass(initPosA, Vec(), 0.0);
	// add a second ball
	int mass1 = addMass(initPosA + Vec(0.0,-2.0,0.0), Vec(1.0,0.0,0.0), massA);
	int spring1 = addSpring(mass0, mass1, stiffnessA, initLengthA, dampingA);

	////////////////////////////////////
	// 2D : Tissu :
	////////////////////////////////////

	////////////////////////////////////
	// 3D : Rseau 3D :
	////////////////////////////////////

	////////////////////////////////////
	// Objects :
	////////////////////////////////////
	// Planes :
	Plane p(Vec(), Vec(0,1,0), Vec(), 0.0);
	planes.push_back(p);
	
	// Balls :
	//Ball b(Vec(0.0,3.0,0.0), 3.0, Vec(),0.0);
	//balls.push_back(b);
}

int Dynamics::addMass(Vec position, Vec velocity, double mass) {
	positions.push_back(position);
	
	velocities.push_back(velocity);
	
	forces.push_back(Vec());
	
	masses.push_back(mass);
	if (mass>0) {
		invMasses.push_back(1/mass);
	} else {
		invMasses.push_back(0.0);
	}

	return positions.size()-1;
}

int Dynamics::addSpring(int mass1, int mass2, double stiffness, double length, double damp) {
	springs.push_back(Spring(mass1, mass2, stiffness, length, damp));
	return springs.size();
}

void Dynamics::setMassPos(int mass, Vec pos) {
	if (mass<0 || mass>=(int)positions.size()){
		printf("Bad mass index !!!\n");
		return;
	}
	positions[mass] = pos;
}

void Dynamics::setMass(int index, double mass) {
	if (index<0 || index>=(int)masses.size()){
		printf("Bad mass index !!!\n");
		return;
	}
	masses[index] = mass;
	if (mass>0) {
		invMasses[index] = 1/mass;
	} else {
		invMasses[index] = 0.0;
	}

}

// Calcule les forces :
void Dynamics::computeForces() {
	///////////////////////////////////////
	//////////////// TO DO ////////////////
	///////////////////////////////////////	
}

// Met  jour les positions :
void Dynamics::updatePositions() {
	///////////////////////////////////////
	//////////////// TO DO ////////////////
	///////////////////////////////////////
}

// Gre les collisions :
void Dynamics::treatCollisions() {
	double rebound = 0.0;
	// Treat collisions for each mass :
	for (int i = 0 ; i < (int)positions.size() ; i++) {
		// Collision with planes :
		for (int j = 0 ; j < (int)planes.size() ; j++) {
			Plane p = planes[j];
			collisionMassPlane(i, j, rebound);
		}
		// Collision with balls :
		for (int j = 0 ; j < (int)balls.size() ; j++) {
			Ball b = balls[j];
			collisionMassBall(i, j, rebound);
		}
	}
}

void Dynamics::collisionMassPlane(int massI, int planeJ, double rebound) {
}

void Dynamics::collisionMassBall(int massI, int ballJ, double rebound)  {
}

// Effectue un pas de temps :
void Dynamics::animate() {
	computeForces();
	updatePositions();
	treatCollisions();
}

// Display dynamic system :
void Dynamics::drawDynamics() {
	// Draw masses :
	for (int i = 0 ; i < (int)positions.size() ; i++) {
		glPushMatrix();
		glTranslatef(positions[i].x, positions[i].y, positions[i].z);
		
		glColor3f(1.0,0.0,.0);
		gluSphere(gluNewQuadric(),radius,60,60);
	
		glPopMatrix();
	}

	// Draw springs :
	glLineWidth(thickness);
	glBegin(GL_LINES);
	for (int i = 0 ; i < (int)springs.size() ; i++) {
		int m1 = springs[i]._m1;
		int m2 = springs[i]._m2;

		Vec pos1 = positions[m1];
		Vec pos2 = positions[m2];
		
		glColor3f(0.0, 1.0, 0.0);
		glVertex3f(pos1.x, pos1.y, pos1.z);
		glVertex3f(pos2.x, pos2.y, pos2.z);
	}
	glEnd();

	// Draw planes :
	double side = 10.0;
	glColor3f(1.0,1.0,.0);
	glBegin(GL_QUADS);
	for (int i = 0 ; i < (int)planes.size() ; i++) {
		Vec dir1 = Vec(planes[i]._normal.y - planes[i]._normal.z, 
			planes[i]._normal.z - planes[i]._normal.x,
			planes[i]._normal.x - planes[i]._normal.y); // toujours perpendiculaire  _normal
		Vec dir2  = cross(dir1, planes[i]._normal);
		dir1.normalize();
		dir2.normalize();
		Vec pos = planes[i]._pos;
		// Grille pour meilleur rendu :
		Vec pt1, pt2, pt3, pt4;
		double subdiv = 10;
		for (int j = 0 ; j < floor(subdiv) ; j++) {
			for (int k = 0 ; k < floor(subdiv) ; k++) {
				pt1 = pos - (j*side/subdiv)*dir1 - (k*side/subdiv)*dir2;
				pt2 = pos + (j*side/subdiv)*dir1 - (k*side/subdiv)*dir2;
				pt3 = pos + (j*side/subdiv)*dir1 + (k*side/subdiv)*dir2;
				pt4 = pos - (j*side/subdiv)*dir1 + (k*side/subdiv)*dir2;
				glVertex3f(pt1.x, pt1.y, pt1.z);
				glVertex3f(pt2.x, pt2.y, pt2.z);
				glVertex3f(pt3.x, pt3.y, pt3.z);
				glVertex3f(pt4.x, pt4.y, pt4.z);
			}
		}
	}
	glEnd();

	// Draw balls :
	glColor3f(1.0,1.0,.0);
	for (int i = 0 ; i < (int)balls.size() ; i++) {
		glPushMatrix();
		glTranslatef(balls[i]._pos.x, balls[i]._pos.y, balls[i]._pos.z);
		
		gluSphere(gluNewQuadric(),balls[i]._rad,60,60);
	
		glPopMatrix();
	}
};