#ifndef DYNAMICS_H_
#define DYNAMICS_H_

#include <vector>
using namespace std;

#include "vec.h"
#include "spring.h"
#include "ball.h"
#include "plane.h"

class Dynamics {
private :
	// Masses :
	vector<Vec> positions; // positions des masses
	vector<Vec> velocities; // vlocit des masses
	vector<double> masses; // masses
	vector<double> invMasses; // inverse des masses

	// Forces :
	vector<Vec> forces; // forces appliques aux masses
	vector<Spring>  springs; // ressorts du systme
	Vec gravity; // gravit exerce sur le systme

	// Paramtres :
	double viscosity; // viscosit du milieu
	double radius; // rayon pour l'affichage des masses
	double thickness; // paisseur pour l'affichage des ressorts

	// Objets de collisions :
	vector<Plane> planes;
	vector<Ball> balls;


public :
	Dynamics(){
		gravity = Vec(0,-10.0,0);
		viscosity = 1.0;
		radius = 0.5;
		thickness = 1.0;
	};

	~Dynamics(){};

	// Cre le systme dynamique :
	void initDynamics();

	// Modifiers :
	int addMass(Vec position, Vec velocity, double mass); // renvoie l'indice de la masse
	int addSpring(int mass1, int mass2, double stiffness, double length, double damp); // renvoie l'indice du ressort
	void setGravity(Vec g) {gravity = g;};
	void setMassPos(int mass, Vec pos);
	void setMass(int index, double mass);

	// Accesseurs :
	const Vec getMassPos(int mass) const {
		if (mass < positions.size()) {
			return positions[mass];
		} else {
			return Vec();
		}
	}
	int getNbSprings() const {return springs.size();}
	const Spring getSpring (int spring) const {return springs[spring];}

	// Calcule les forces :
	void computeForces();
	// Met  jour les positions :
	void updatePositions();
	// Gre les collisions :
	void treatCollisions();
	void collisionMassPlane(int massI, int planeJ, double rebound);
	void collisionMassBall(int massI, int ballJ, double rebound );
	// Effectue un pas de temps :
	void animate();

	// Display dynamic system :
	void drawDynamics();

	

};

#endif