#ifndef VEC_H_
#define VEC_H_

#include "math.h"

class Vec {
public :
	double x;
	double y;
	double z;

public :
	Vec() : x(0.0), y(0.0), z(0.0) {
	};

	Vec(double x0, double y0, double z0) : x(x0), y(y0), z(z0) {
	};

	Vec(const Vec& v) : x(v.x), y(v.y), z(v.z) {
	};

	~Vec(){
	};
	
	// Oprateurs :
	double operator* (const Vec& v) {
		return x*v.x + y*v.y + z*v.z;
	}

	friend Vec cross(const Vec &a, const Vec &b) {
      return Vec(a.y*b.z - a.z*b.y,
		 a.z*b.x - a.x*b.z,
		 a.x*b.y - a.y*b.x);
    }

	double norm() const { 
		return sqrt(x*x + y*y + z*z); 
	}

	double normalize() {
		const double n = norm();
		x /= n;
		y /= n;
		z /= n;
		return n;
    }

	friend Vec operator* (double f, const Vec& v) {
		return Vec(f*v.x, f*v.y, f*v.z);
	}
	
	Vec operator* (double f) {
		return Vec(f*x, f*y, f*z);
	}

	Vec operator- (const Vec& v) {
		return Vec(x-v.x, y-v.y, z-v.z);
	}
	
	Vec operator+ (const Vec& v) {
		return Vec(x+v.x, y+v.y, z+v.z);
	}
};

#endif